/***************************************************************************
wancfglib_chan_def.cpp  -  description
-------------------
begin                : Tue Mar 16 2004
copyright            : (C) 2004 by David Rokhvarg
email                : davidr@sangoma.com
***************************************************************************/

/***************************************************************************
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
***************************************************************************/


#include "stdafx.h"
#include "wancfglib_chan_def.h"

#define DBG_LIST_ELEMENT_CHAN_DEF  0

wancfglib_chan_def::wancfglib_chan_def()
{
	Debug(DBG_LIST_ELEMENT_CHAN_DEF, ("%s()\n", __FUNCTION__));

	memset(&if_data, 0x00, sizeof(if_data));
	memset(&link_defs, 0x00, sizeof(link_defs));

	tmp_conf = 0;

	strlcpy(if_data.name, CHAN_NOT_CONFIGURED, WAN_IFNAME_SZ);

	WANCFGLIB_MALLOC(wanif_conf_t, if_data.chanconf, sizeof(wanif_conf_t));
	if(if_data.chanconf == NULL){
		ERR_DBG_OUT(("Memory allocation failed 'wanif_conf_t' object!!\n"));
	}

	if_data.chanconf->active_ch = ENABLE_ALL_CHANNELS;
	if_data.chanconf->config_id = PROTOCOL_NOT_SET;
	if_data.chanconf->mtu = 1600;
	if_data.chanconf->mc = WANOPT_NO;
	if_data.chanconf->true_if_encoding = WANOPT_NO;
	if_data.chanconf->if_down = WANOPT_NO;

	/////////////////////////////////////////////////////////////
	WANCFGLIB_MALLOC(wandev_conf_t, link_defs.linkconf, sizeof(wandev_conf_t));
	if (link_defs.linkconf == NULL){
		ERR_DBG_OUT(("Memory allocation failed for 'wandev_conf_t'!!\n"));
	}

	link_defs.linkconf->card_type = NOT_SET;
	link_defs.linkconf->config_id = PROTOCOL_NOT_SET;
	link_defs.linkconf->magic = ROUTER_MAGIC;
	link_defs.linkconf->mtu = 1500;
	link_defs.linkconf->udp_port = 9000;
	link_defs.linkconf->ttl = 255;
	link_defs.linkconf->S514_CPU_no[0] = 'A';
	link_defs.linkconf->comm_port = WANOPT_PRI;
	link_defs.linkconf->auto_hw_detect = WANOPT_YES;
	link_defs.linkconf->electrical_interface = WANOPT_V35;

	DebugV("%s(): this ptr: 0x%p\n", __FUNCTION__, this);
}

wancfglib_chan_def::~wancfglib_chan_def()
{
	DebugV("%s(): this ptr: 0x%p\n", __FUNCTION__, this);
	Debug(DBG_LIST_ELEMENT_CHAN_DEF, ("%s(): deleting channel: %s, next_objects_list.size(): %d\n", 
		__FUNCTION__, if_data.name, next_objects_list.size()));	

	if(tmp_conf) free(tmp_conf);

	//first deleted all objects which are in 'next_objects_list'
	for(unsigned int index = 0; index < next_objects_list.size(); ++index){
		Debug(DBG_LIST_ELEMENT_CHAN_DEF, ("in next_objects_list deleting channel: %s\n", next_objects_list.at(index)->if_data.name));	
		delete next_objects_list.at(index);
	}

	if (if_data.conf)			free(if_data.conf);
	if (if_data.conf_profile)	free(if_data.conf_profile);
	if (if_data.addr)			free(if_data.addr);
	if (if_data.usedby)			free(if_data.usedby);
	if (if_data.label)			free(if_data.label);
	if (if_data.virtual_addr)	free(if_data.virtual_addr);
	if (if_data.real_addr)		free(if_data.real_addr);
	if (if_data.chanconf)		free(if_data.chanconf);

	/////////////////////////////////////
	Debug(DBG_LIST_ELEMENT_CHAN_DEF, ("Freeing Link %s\n", link_defs.name));

	if (link_defs.conf) free(link_defs.conf);
	if (link_defs.descr) free(link_defs.descr);

	if (link_defs.linkconf){
		if (link_defs.linkconf->data){
			free(link_defs.linkconf->data);
		}
		free(link_defs.linkconf);
	}
}

int wancfglib_chan_def::compare(wancfglib_chan_def* element)
{
	Debug(DBG_LIST_ELEMENT_CHAN_DEF, ("%s(wancfglib_chan_def* element)\n", __FUNCTION__));

	//convert to decimal values. string comparisons do not work sometimes
	//because of invisible characters in strings.
	int existing = atoi(wancfglib_remove_spaces_in_int_string(if_data.addr));
	int new_el = atoi(wancfglib_remove_spaces_in_int_string(((wancfglib_chan_def*)element)->if_data.addr));
	int rc;

	Debug(DBG_LIST_ELEMENT_CHAN_DEF,("DEC element existing: %d, DEC element for comparison: %d\n",
		existing, new_el));

	if(existing  < new_el){
		rc = -1;
	}else if(existing  > new_el){
		rc = 1;
	}else{
		rc = 0;
	}

	Debug(DBG_LIST_ELEMENT_CHAN_DEF, ("comparison result %d\n", rc));
	return rc;
}

int wancfglib_chan_def::compare(char* comparator)
{
	Debug(DBG_LIST_ELEMENT_CHAN_DEF, ("%s(char* comparator)\n", __FUNCTION__));
	Debug(DBG_LIST_ELEMENT_CHAN_DEF, ("if_data.addr: %s, comparator: %s\n", if_data.addr, comparator));
	return strcmp(if_data.addr, comparator);
}

//the comparator is char string in this case
char* wancfglib_chan_def::get_comparator()
{
	Debug(DBG_LIST_ELEMENT_CHAN_DEF, ("%s()\n", __FUNCTION__));
	return if_data.addr;
}

void wancfglib_chan_def::set_comparator(char* comparator)
{
	Debug(DBG_LIST_ELEMENT_CHAN_DEF, ("%s()\n", __FUNCTION__));
	if_data.addr = strdup(comparator);
	Debug(DBG_LIST_ELEMENT_CHAN_DEF, ("%s(): if_data.addr ptr: 0x%p\n", __FUNCTION__, if_data.addr));
}

