/***************************************************************************
                          wancfglib_conf_file_reader.h  -  description
                             -------------------
    begin                : Thu Mar 4 2004
    copyright            : (C) 2004 by David Rokhvarg
    email                : davidr@sangoma.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/* Structure diagram.
 * Notes:
 *  1.	All LIP interfaces belonging to the same HW interface will
 *		have a pointer back to the HW interface where 'profile'
 *		configuration will be stored. 
 *		For example, each DLCI in Frame Relay will have the pointer
 *		to the same 'global profile' of FR configuration.
 *		And, each interface will have it's own configuration:
 *		CIR, and so on.
 *  2.	Each LIP interface in turn, may be 'used by' as STACK,
 *		and the same logic apply recursively.
 *     
+---------------------------+
|conf_file_reader(CFR)|		|
|---------------------		|				Hardware Interfaces					LIP Interfaces
|							|				-------------------					--------------
|link_def_t	link_defs		|			    
|objects_list obj_list		|
+---------------------------+
		|								|---------------------------|		
		|								|list_element_chan_def|		|
		+-------------------------------|----------------------		|
		|								|(w1g1)						|
		|								|chan_def_t if_data			|
		|								|link_def_t link_defs	|
		|								|objects_list obj_list		|---+
		|								|---------------------------|	|
		|																|	|---------------------------|
		|																|	|list_element_chan_def|		|
		|																+---|----------------------		|
		|								|---------------------------|	|	|(DLCI 16)					|
		|								|list_element_chan_def|		|	|	|chan_def_t if_data			|
		|								|---------------------		|	|	|link_def_t nxt_lvl_profile	|
		+-------------------------------|(w1g1)						|	|	|objects_list obj_list		|
										|chan_def_t if_data			|	|	|---------------------------|
										|link_def_t nxt_lvl_profile	|	|
										|objects_list obj_list		|	|
										|---------------------------|	|
 																		|	|---------------------------|
																		|	|list_element_chan_def|		|
																		+---|----------------------		|
																			|(DLCI 17)					|
																			|chan_def_t if_data			|
																			|link_def_t nxt_lvl_profile	|
																			|objects_list obj_list		|
																			|---------------------------|
*/

#ifndef WANCFGLIB_CONF_FILE_READER_H
#define WANCFGLIB_CONF_FILE_READER_H

#include "wancfglib_chan_def.h"

/** Reads wanpipe#.conf file.
 *  Allocates configuration structures and linked lists.
 */
class wancfglib_conf_file_reader : public wancfglib_chan_def {

private:
	int		verbose;		/* verbosity level */
	char	conf_file_full_path[MAX_PATH_LENGTH];
	FILE	*conf_file_ptr;

	int get_wanpipe_cfg_directory();
	int parse_conf_file(char* fname);
	int read_conf_record (FILE* file, char* key, int max_len);
	char* read_conf_section (FILE* file, char* section);
	int set_conf_param (char* key, char* val, key_word_t* dtab, void* conf, int max_len);
	int read_devices_section(FILE* file);
	int read_wanpipe_section(FILE* file);
	int read_interfaces_section(FILE* file, wancfglib_chan_def & parent_chan_def_obj, char* parent_dev_name, int* current_line_number);

	int configure_link (link_def_t* def, char init);
	int configure_chan (chan_def_t* chandef, int id);

	//this member initialized as follows:
	//1. If "New" wanpipe - user selects wanpipe number.
	//2. If "Edit existing" - wanpipe number is given (read from the conf file).
	int wanpipe_number;

	char wanpipe_cfg_dir[MAX_PATH_LENGTH];

public:

	wancfglib_conf_file_reader(int wanpipe_number);
	virtual ~wancfglib_conf_file_reader();

	//Call this just after creating the object.
	//If non-zero return code, the object can *not* be used!
	int read_conf_file();

	int get_port_cfg(port_cfg_t	*port_cfg);
	int configure_hw_interfaces();
};

#endif//WANCFGLIB_CONF_FILE_READER_H
