//////////////////////////////////////////////////////////////////////
// wancfglib_utils.cpp: implementation of global functions in 
//						wancfglib.dll.
//
// Author	:	David Rokhvarg	<davidr@sangoma.com>
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "wancfglib.h"
#include "wanconfig.h"

#define DBG_WANCFGLIB_UTILS 1
#define WANCFGLIB_INFO	printf

static unsigned int wancfglib_get_active_channels(int channel_flag, int start_channel, int stop_channel);


int wancfglib_get_used_by_integer_value(char* used_by_str)
{
	if(strcmp(used_by_str, "WANPIPE") == 0){
		return WANPIPE;
	}else if(strcmp(used_by_str, "API") == 0){
		return API;
	}else if(strcmp(used_by_str, "TDM_API") == 0){
		return TDM_API;
	}else if(strcmp(used_by_str, "BRIDGE") == 0){
		return BRIDGE;
	}else if(strcmp(used_by_str, "BRIDGE_NODE") == 0){
		return BRIDGE_NODE;
	}else if(strcmp(used_by_str, "SWITCH") == 0){
		return WP_SWITCH;
	}else if(strcmp(used_by_str, "STACK") == 0){
		return STACK;
	}else if(strcmp(used_by_str, "ANNEXG") == 0){
		return ANNEXG;
	}else if(strcmp(used_by_str, "TDM_VOICE") == 0){
		return TDM_VOICE;
	}else if(strcmp(used_by_str, "TTY") == 0){
		return TTY;
//	}else if(strcmp(used_by_str, "PPPoE") == 0){
//		return PPPoE;
	}else if(strcmp(used_by_str, "NETGRAPH") == 0){
		return WP_NETGRAPH;
	}else if(strcmp(used_by_str, "TDM_VOICE_API") == 0){
		return TDM_VOICE_API;

	}else if(strcmp(used_by_str, "TDM_SPAN_VOICE_API") == 0){
		return TDM_SPAN_VOICE_API;
	}else if(strcmp(used_by_str, "TDM_CHAN_VOICE_API") == 0){
		return TDM_CHAN_VOICE_API;
	}else if(strcmp(used_by_str, "API_LEGACY") == 0){
		return API_LEGACY;

	}else{
		return -1;
	}
}

int wancfglib_get_config_id_from_profile(char* config_id)
{
	if (!strcmp(config_id, "fr")){
		return WANCONFIG_MFR;
	}else if(!strcmp(config_id, "ppp")){
		return WANCONFIG_MPPP;
	}else if(!strcmp(config_id, "chdlc")){
		return WANCONFIG_MPCHDLC;
	}else if(!strcmp(config_id, "tty")){
		return WANCONFIG_TTY;
	}else if(!strcmp(config_id, "lip_lapb")){
		return WANCONFIG_LAPB;
	}else if(!strcmp(config_id, "lip_atm")){
		return WANCONFIG_LIP_ATM;
	}else{
		return PROTOCOL_NOT_SET;
	}
}

/*============================================================================
 * TE1
 * Parse active channel string.
 *
 * Return ULONG value, that include 1 in position `i` if channels i is active.
 */
unsigned int wancfglib_parse_active_channel(char* val)
{
#define SINGLE_CHANNEL	0x2
#define RANGE_CHANNEL	0x1
	int channel_flag = 0;
	char* ptr = val;
	int channel = 0, start_channel = 0;
	unsigned int tmp = 0;

	if (strcmp(val,"ALL") == 0)
		return ENABLE_ALL_CHANNELS;
	
	if (strcmp(val,"0") == 0)
		return 0;

	while(*ptr != '\0') {
       		//WANCFGLIB_INFO("\nMAP DIGIT %c\n", *ptr);
		if (isdigit(*ptr)) {
			channel = strtoul(ptr, &ptr, 10);
			channel_flag |= SINGLE_CHANNEL;
		} else {
			if (*ptr == '-') {
				channel_flag |= RANGE_CHANNEL;
				start_channel = channel;
			} else {
				tmp |= wancfglib_get_active_channels(channel_flag, start_channel, channel);
				channel_flag = 0;
			}
			ptr++;
		}
	}
	if (channel_flag){
		tmp |= wancfglib_get_active_channels(channel_flag, start_channel, channel);
	}

	return tmp;
}

static unsigned int wancfglib_get_active_channels(int channel_flag, int start_channel, int stop_channel)
{
	int i = 0;
	unsigned int tmp = 0, mask = 0;

	/* If the channel map is set to 0 then
	 * stop_channel will be zero. In this case just return
	 * 0 */
	if (stop_channel < 1) {
        	return 0;
	}

	if ((channel_flag & (SINGLE_CHANNEL | RANGE_CHANNEL)) == 0)
		return tmp;
	if (channel_flag & RANGE_CHANNEL) { /* Range of channels */
		for(i = start_channel; i <= stop_channel; i++) {
			mask = 1 << (i - 1);
			tmp |=mask;
		}
	} else { /* Single channel */ 
		mask = 1 << (stop_channel - 1);
		tmp |= mask; 
	}
	return tmp;
}


//convert int definition of a protocol to string
char* wancfglib_get_protocol_string(int protocol)
{
	static char protocol_name[MAX_PATH_LENGTH];
#if 0
	conf_file_reader* local_cfr = (conf_file_reader*)global_conf_file_reader_ptr;
#endif

	switch(protocol)
	{
	case PROTOCOL_NOT_SET:
		snprintf((char*)protocol_name, MAX_PATH_LENGTH, "Not Configured");
		break;

	case NO_PROTOCOL_NEEDED:
		//special case, when no protocol name actually needed
		protocol_name[0] = '\0';
		break;

	case WANCONFIG_X25:
		snprintf((char*)protocol_name, MAX_PATH_LENGTH, "X25");
		break;

	case WANCONFIG_FR:
		snprintf((char*)protocol_name, MAX_PATH_LENGTH, "Frame Relay");
		break;

	case WANCONFIG_PPP:
		snprintf((char*)protocol_name, MAX_PATH_LENGTH, "PPP");
		break;

	case WANCONFIG_CHDLC:
		snprintf((char*)protocol_name, MAX_PATH_LENGTH, "CHDLC");
		break;

	case WANCONFIG_BSC:
		snprintf((char*)protocol_name, MAX_PATH_LENGTH, "BiSync Streaming");
		break;

	case WANCONFIG_HDLC:
		//used with CHDLC firmware
		snprintf((char*)protocol_name, MAX_PATH_LENGTH, "HDLC Streaming");
		break;

	case WANCONFIG_MPPP://and WANCONFIG_MPROT too
		//snprintf((char*)protocol_name, MAX_PATH_LENGTH, "Multi Port PPP");
		snprintf((char*)protocol_name, MAX_PATH_LENGTH, "PPP");
		break;

	case WANCONFIG_BITSTRM:
		snprintf((char*)protocol_name, MAX_PATH_LENGTH, "Bit Stream");
		break;

	case WANCONFIG_EDUKIT:
		snprintf((char*)protocol_name, MAX_PATH_LENGTH, "WAN EduKit");
		break;

	case WANCONFIG_SS7:
		snprintf((char*)protocol_name, MAX_PATH_LENGTH, "SS7");
		break;

	case WANCONFIG_BSCSTRM:
		snprintf((char*)protocol_name, MAX_PATH_LENGTH, "Bisync Streaming Nasdaq");
		break;

	case WANCONFIG_MFR:
		//snprintf((char*)protocol_name, MAX_PATH_LENGTH, "Multi-Port Frame Relay");
		snprintf((char*)protocol_name, MAX_PATH_LENGTH, "Frame Relay");
		break;
#if 0
	case WANCONFIG_ADSL:

		if(local_cfr != NULL){
			switch(local_cfr->link_defs->sub_config_id)
			{
			case WANCONFIG_MPPP:
				snprintf((char*)protocol_name, MAX_PATH_LENGTH, "PPP (on ADSL)");
				break;

			case PROTOCOL_IP:
				snprintf((char*)protocol_name, MAX_PATH_LENGTH, ADSL_IP_STR);
				break;

			case PROTOCOL_ETHERNET:
				snprintf((char*)protocol_name, MAX_PATH_LENGTH, ADSL_ETHERNET_STR);
				break;

			default:
				snprintf((char*)protocol_name, MAX_PATH_LENGTH, "ADSL");
			}

		}else{
			snprintf((char*)protocol_name, MAX_PATH_LENGTH, "ADSL");
		}
		break;
#endif

	case WANCONFIG_SDLC:
		snprintf((char*)protocol_name, MAX_PATH_LENGTH, "SDLC");
		break;

	case WANCONFIG_ATM:
		snprintf((char*)protocol_name, MAX_PATH_LENGTH, "ATM");
		break;

	case WANCONFIG_POS:
		snprintf((char*)protocol_name, MAX_PATH_LENGTH, "Point-of-Sale");
		break;

	case WANCONFIG_AFT:
		snprintf((char*)protocol_name, MAX_PATH_LENGTH, "AFT(TE1)");
		break;

	case WANCONFIG_AFT_TE3:
		snprintf((char*)protocol_name, MAX_PATH_LENGTH, "AFT(TE3)");
		break;

	case WANCONFIG_DEBUG:
		snprintf((char*)protocol_name, MAX_PATH_LENGTH, "Real Time Debugging");
		break;

	case WANCONFIG_ADCCP:
		snprintf((char*)protocol_name, MAX_PATH_LENGTH, "Special HDLC LAPB");
		break;

	case WANCONFIG_MLINK_PPP:
		snprintf((char*)protocol_name, MAX_PATH_LENGTH, "Multi-Link PPP");
		break;

	case WANCONFIG_GENERIC:
		snprintf((char*)protocol_name, MAX_PATH_LENGTH, "WANPIPE Generic driver");
		break;

	case WANCONFIG_MPCHDLC:
		//snprintf((char*)protocol_name, MAX_PATH_LENGTH, "Multi-Port CHDLC");
		snprintf((char*)protocol_name, MAX_PATH_LENGTH, "CHDLC");
		break;

	case WANCONFIG_TTY:
		snprintf((char*)protocol_name, MAX_PATH_LENGTH, "TTY");
		break;

	case PROTOCOL_TDM_VOICE:
		snprintf((char*)protocol_name, MAX_PATH_LENGTH, "TDM Voice");
		break;

	case PROTOCOL_TDM_VOICE_API:
		snprintf((char*)protocol_name, MAX_PATH_LENGTH, "TDM Voice API");
		break;

	case WANCONFIG_LAPB:
		snprintf((char*)protocol_name, MAX_PATH_LENGTH, "HDLC LAPB");
		break;

	case WANCONFIG_LIP_ATM:
		snprintf((char*)protocol_name, MAX_PATH_LENGTH, "ATM (LIP)");
		break;

	default:
		ERR_DBG_OUT(("Invalid protocol: %d\n", protocol));
		snprintf((char*)protocol_name, MAX_PATH_LENGTH, INVALID_PROTOCOL);
		break;
	}

	return (char*)protocol_name;
}

//caller must be sure 'input' string is actually an integer.
char* wancfglib_remove_spaces_in_int_string(char* input)
{
	char output [LEN_OF_DBG_BUFF];

	//remove spaces and zero terminate.
	snprintf(output, LEN_OF_DBG_BUFF, "%d", atoi(input));
	//copy back to caller's buffer
	strlcpy(input, output, MAX_PATH_LENGTH);

	return input;
}

int wancfglib_config_id_to_annexg_id(int config_id)
{
	switch(config_id)
	{
	case WANCONFIG_MFR:
		return ANNEXG_FR;
	case WANCONFIG_MPPP:
		return ANNEXG_PPP;
	case WANCONFIG_MPCHDLC:
		return ANNEXG_CHDLC;
	case WANCONFIG_LAPB:
		return ANNEXG_LIP_LAPB;
	case WANCONFIG_LIP_ATM:
		return ANNEXG_LIP_ATM;
	default:
		ERR_DBG_OUT(("Invalid config_id: %d!\n", config_id));
		return 0;
	}
}

int wancfglib_print_wanif_conf_t_structure(wanif_conf_t *wanif_conf)
{
	WANCFGLIB_INFO("Operation Mode\t: %s\n", wanif_conf->usedby);
	WANCFGLIB_INFO("Timeslot BitMap\t: 0x%08X\n", wanif_conf->active_ch);
	WANCFGLIB_INFO("Line Mode\t: %s\n",
				(wanif_conf->hdlc_streaming == WANOPT_YES ? "HDLC":"BitStream"));
	WANCFGLIB_INFO("MTU\\MRU\t\t: %d\n", wanif_conf->mtu);

	return 0;
}

int wancfglib_print_sdla_fe_cfg_t_structure(sdla_fe_cfg_t *sdla_fe_cfg)
{
	WANCFGLIB_INFO("\n################################################\n");
	WANCFGLIB_INFO("MEDIA\t\t: %s\n", MEDIA_DECODE(sdla_fe_cfg));
	if(FE_MEDIA(sdla_fe_cfg) == WAN_MEDIA_T1 || FE_MEDIA(sdla_fe_cfg) == WAN_MEDIA_E1){
		WANCFGLIB_INFO("Line CODE\t: %s\n", LCODE_DECODE(sdla_fe_cfg));
		WANCFGLIB_INFO("Framing\t\t: %s\n", FRAME_DECODE(sdla_fe_cfg));
		WANCFGLIB_INFO("Clock Mode\t: %s\n", TECLK_DECODE(sdla_fe_cfg));
		WANCFGLIB_INFO("Clock Reference port: %d (0 - not used)\n", FE_REFCLK(sdla_fe_cfg));
		WANCFGLIB_INFO("Signalling Insertion Mode: %s\n", TE1SIG_DECODE(sdla_fe_cfg));

		WANCFGLIB_INFO("High Impedance Mode: %s\n",
			(FE_HIMPEDANCE_MODE(sdla_fe_cfg) == WANOPT_YES ? "Yes":"No"));
		WANCFGLIB_INFO("FE_RX_SLEVEL: %d\n", FE_RX_SLEVEL(sdla_fe_cfg));
	}//if()
	WANCFGLIB_INFO("TDMV LAW\t: %s\n", (FE_TDMV_LAW(sdla_fe_cfg) == WAN_TDMV_MULAW ?"MuLaw":"ALaw"));
	WANCFGLIB_INFO("TDMV SYNC\t: %d\n", FE_NETWORK_SYNC(sdla_fe_cfg));

	switch(FE_MEDIA(sdla_fe_cfg))
	{
	case WAN_MEDIA_T1:
		WANCFGLIB_INFO("LBO\t\t: %s\n", LBO_DECODE(sdla_fe_cfg));
		break;
	case WAN_MEDIA_E1:
		break;
	default:
		break;
	}
	return 0;
}

int wancfglib_print_port_cfg_structure(port_cfg_t *port_cfg)
{
	wandev_conf_t		*wandev_conf = &port_cfg->wandev_conf;
	sdla_fe_cfg_t		*sdla_fe_cfg = &wandev_conf->fe_cfg;
	wanif_conf_t		*wanif_conf = &port_cfg->if_cfg[0];
	unsigned int		i;

	WANCFGLIB_INFO("\n================================================\n");

	WANCFGLIB_INFO("Card Type\t: %s(%d)\n", SDLA_DECODE_CARDTYPE(wandev_conf->card_type),
		wandev_conf->card_type);/* Sangoma Card type - S514, S518 or AFT.*/

	WANCFGLIB_INFO("Number of TimeSlot Groups: %d\n", port_cfg->num_of_ifs);

	WANCFGLIB_INFO("MTU\t\t: %d\n",	wandev_conf->mtu);

	wancfglib_print_sdla_fe_cfg_t_structure(sdla_fe_cfg);

	for(i = 0; i < port_cfg->num_of_ifs; i++){
		WANCFGLIB_INFO("\n************************************************\n");
		WANCFGLIB_INFO("Configration of Group Number %d:\n", i);
		wancfglib_print_wanif_conf_t_structure(&wanif_conf[i]);
	}
	return 0;
}

int wancfglib_active_channels_bitmap_to_str(unsigned int chan_map, int start_channel, int stop_channel, char* chans_str)
{
	int i;
	unsigned char enabled = 0;
	unsigned char  first_chan = 0;
	unsigned char last_chan = 0;
	int str_len = 0;
	
	for(i = start_channel; i <= stop_channel; i++) {
		if (chan_map & (1 <<( i-1))) {
			if (!enabled) {
				enabled = 1;
				if (str_len > 0) {
					str_len += sprintf(&chans_str[str_len], ".");
				}
				first_chan = i;
			}
			if (last_chan < i) {
				last_chan = i;
			}
		} else {
			if (enabled) {
				enabled = 0;
				if (last_chan > first_chan) {
					str_len += sprintf(&chans_str[str_len], "%d-%d", first_chan, last_chan);
				} else {
					str_len += sprintf(&chans_str[str_len], "%d", first_chan);
				}
			}
		}
	}
	if (enabled) {
		if (last_chan > first_chan) {
			str_len += sprintf(&chans_str[str_len], "%d-%d", first_chan, last_chan);
		} else {
			str_len += sprintf(&chans_str[str_len], "%d", first_chan);
		}
	}
	return 0;
}

