//////////////////////////////////////////////////////////////////////
// wp_hwprobe.cpp: implementation of the wp_hwprobe class.
//
// Author	:	David Rokhvarg	<davidr@sangoma.com>
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "wp_hwprobe.h"
#include "sangoma_port.h"
#include <iostream>	//for cout
#include <sstream>	//for stringstream

using namespace wanrouter;

#define DBG_HWPROBE if(0)printf
#define HWPROBE_INFO	printf

template <class T>
inline std::string to_string (const T& t)
{
	std::stringstream ss;
	ss << t;
	return ss.str();
}

////////////////////////////////////////////////////////////////////////////////////////
// Constructor/Destructor
////////////////////////////////////////////////////////////////////////////////////////

wp_hwprobe::wp_hwprobe(void)
{
	memset(&wp_hwprobe_priv, 0x00, sizeof(wp_hwprobe_priv));
	cVerbose = 0;
}

wp_hwprobe::~wp_hwprobe(void)
{
}

////////////////////////////////////////////////////////////////////////////////////////
// Private functions
////////////////////////////////////////////////////////////////////////////////////////

int wp_hwprobe::add_port(hardware_info_t & hardware_info)
{
	DBG_HWPROBE("card_model\t\t: %s (0x%08X)\n", 
		SDLA_ADPTR_NAME(hardware_info.card_model), hardware_info.card_model);
	DBG_HWPROBE("firmware_version\t: 0x%X\n", hardware_info.firmware_version);
	DBG_HWPROBE("pci_bus_number\t\t: %d\n", hardware_info.pci_bus_number);
	DBG_HWPROBE("pci_slot_number\t\t: %d\n", hardware_info.pci_slot_number);
	DBG_HWPROBE("port_number\t\t: %d\n", hardware_info.port_number);
	DBG_HWPROBE("max_hw_ec_chans\t\t: %d\n", hardware_info.max_hw_ec_chans);

	hw_info_vec.push_back(hardware_info);

	switch(hardware_info.card_model)
	{
	case S518_ADPTR_1_CPU_ADSL:
		break;
	case A101_ADPTR_1TE1:
	case A101_ADPTR_2TE1:
		HW_CNTR.aft101_adapters++;
		break;
	case A104_ADPTR_4TE1:
		HW_CNTR.aft104_adapters++;
		break;
	case A108_ADPTR_8TE1:
		HW_CNTR.aft108_adapters++;
		break;
	case A200_ADPTR_ANALOG:
	case A400_ADPTR_ANALOG:
		HW_CNTR.aft200_adapters++;
		break;
	case AFT_ADPTR_ISDN:
		HW_CNTR.aft_isdn_adapters++;
		break;
	case AFT_ADPTR_56K:
		HW_CNTR.aft_56k_adapters++;
		break;
	case AFT_ADPTR_2SERIAL_V35X21:
	case AFT_ADPTR_4SERIAL_V35X21:
	case AFT_ADPTR_2SERIAL_RS232:
	case AFT_ADPTR_4SERIAL_RS232:
		HW_CNTR.aft_serial_adapters++;
		break;
	case AFT_ADPTR_A600:	//"AFT-B600"
		HW_CNTR.aft_a600_adapters++;
		break;
	case AFT_ADPTR_FLEXBRI:	//"AFT-B700"
		HW_CNTR.aft_a700_adapters++;
		break;
	default:
		HWPROBE_INFO("Warning: unsupported card type: %s (%d)\n",
			SDLA_ADPTR_NAME(hardware_info.card_model), hardware_info.card_model);
		break;
	}

	return 0;
}

//
//Returns:	-1 - if error.
//			number of cards, starting from zero.
int wp_hwprobe::scan_for_sangoma_ports()
{
	uint16_t port_counter = 0, wp_ind;
	hardware_info_t	hardware_info;
	sangoma_port sng_port;

	hw_info_vec.empty();

	for(wp_ind = 0; wp_ind < (WP_MAX_PORTS); wp_ind++){

		if(sng_port.init(wp_ind)){
			continue;
		}

		if(sng_port.get_hardware_info(&hardware_info)){
			port_counter = -1;
			break;
		}

		if(add_port(hardware_info) == 0){
			port_counter++;
		}

		sng_port.cleanup();

	}//for()

	return port_counter;
}

void wp_hwprobe::wp_get_port_count_summary(std::string & strPortSummary)
{
	char m[200];

	//note that we report Port count, NOT Card count
	strPortSummary = "\nPort Cnt: ";

	if (HW_CNTR.s518_adapters){
		wp_snprintf(m, sizeof(m), "S518=%d ", HW_CNTR.s518_adapters);
		strPortSummary += m;
	}
	if (HW_CNTR.aft101_adapters){
		wp_snprintf(m, sizeof(m), "A101-2=%d ", HW_CNTR.aft101_adapters);
		strPortSummary += m;
	}
	if (HW_CNTR.aft104_adapters){
		wp_snprintf(m, sizeof(m), "A104=%d ", HW_CNTR.aft104_adapters);
		strPortSummary += m;
	}
	if (HW_CNTR.aft108_adapters){
		wp_snprintf(m, sizeof(m), "A108=%d ", HW_CNTR.aft108_adapters);
		strPortSummary += m;
	}
	if (HW_CNTR.aft200_adapters){
		wp_snprintf(m, sizeof(m), "A200=%d ", HW_CNTR.aft200_adapters);
		strPortSummary += m;
	}
	if (HW_CNTR.aft_56k_adapters){
		wp_snprintf(m, sizeof(m), "A056=%d ", HW_CNTR.aft_56k_adapters);
		strPortSummary += m;
	}
	if (HW_CNTR.aft_isdn_adapters){
		wp_snprintf(m, sizeof(m), "A500=%d ", HW_CNTR.aft_isdn_adapters);
		strPortSummary += m;
	}
	if (HW_CNTR.aft_a700_adapters){
		wp_snprintf(m, sizeof(m), "B700=%d ", HW_CNTR.aft_a700_adapters);
		strPortSummary += m;
	}
	if (HW_CNTR.aft_serial_adapters){
		wp_snprintf(m, sizeof(m), "A14x=%d ", HW_CNTR.aft_serial_adapters);
		strPortSummary += m;
	}
	if (HW_CNTR.aft300_adapters){
		wp_snprintf(m, sizeof(m), "A300=%d ", HW_CNTR.aft300_adapters);
		strPortSummary += m;
	}
	if (HW_CNTR.usb_adapters){
		wp_snprintf(m, sizeof(m), "U100=%d ", HW_CNTR.usb_adapters);
		strPortSummary += m;
	}
	if (HW_CNTR.aft_a600_adapters){
		wp_snprintf(m, sizeof(m), "B600=%d ", HW_CNTR.aft_a600_adapters);
		strPortSummary += m;
	}
	if (HW_CNTR.aft_b601_adapters){
		wp_snprintf(m, sizeof(m), "B601=%d ", HW_CNTR.aft_b601_adapters);
		strPortSummary += m;
	}
	wp_snprintf(m, sizeof(m), "\n");
	strPortSummary += m;
}

//For Analog Ports, show which modules installed on the HW.
int wp_hwprobe::get_remora_modules(hardware_info_t & hwinfo, std::string & strModuleProbeList)
{
	if(hwinfo.fxo_map || hwinfo.fxs_map){

		char cTemp[100];
		std::string strTemp;

		for(int i = 1 /* bit zero is never set */; i <= MAX_REMORA_MODULES /* 24 */; i++){

			if( hwinfo.fxo_map & (1 << i) ){
				wp_snprintf(cTemp, sizeof(cTemp), "%02d:FXO\n", i);
				strTemp += cTemp;
			}
			
			if( hwinfo.fxs_map & (1 << i) ){
				wp_snprintf(cTemp, sizeof(cTemp), "%02d:FXS\n", i);
				strTemp += cTemp;
			}
			
			if( !(hwinfo.fxo_map & (1 << i)) &&  !(hwinfo.fxs_map & (1 << i))){
				wp_snprintf(cTemp, sizeof(cTemp), "%02d:EMPTY\n", i);
				strTemp += cTemp;
			}
		}

		strModuleProbeList += strTemp;
	}

	if(hwinfo.bri_modtype){
		char cTemp[100];

		wp_snprintf(cTemp, sizeof(cTemp), "%02d:%s\n",
			hwinfo.port_number, WP_BRI_DECODE_MOD_TYPE(hwinfo.bri_modtype));

		strModuleProbeList += cTemp;
	}

	return 0;
}

int wp_hwprobe::run(std::string & strProbeList)
{
	int	port_counter;
	std::string strPortSummary;
	std::vector<hardware_info_t>::iterator hwIterator;
	std::string strAnalogModules;

	memset(&wp_hwprobe_priv, 0x00, sizeof(wp_hwprobe_priv));

	hw_info_vec.clear();

	if(scan_for_sangoma_ports() < 0){
		return 1;
	}

	hwIterator = hw_info_vec.begin();

	port_counter = 0;

	while(hwIterator != hw_info_vec.end()){

		port_counter++;
		//line number
		wp_snprintf(&HW_STR[strlen(HW_STR)],
			sizeof(HW_STR) - strlen(HW_STR), 
			"%d. ", port_counter);

		//line content
		wp_snprintf(&HW_STR[strlen(HW_STR)],
			sizeof(HW_STR) - strlen(HW_STR),
			WP_HW_PROBE_FORMAT, 
			SDLA_ADPTR_NAME(hwIterator->card_model),
			hwIterator->pci_slot_number,
			hwIterator->pci_bus_number,
			hwIterator->port_number,
			hwIterator->max_hw_ec_chans,
			hwIterator->firmware_version);

		if(cVerbose){
			strAnalogModules.clear();
			get_remora_modules(*hwIterator, strAnalogModules);

			wp_snprintf(&HW_STR[strlen(HW_STR)],
				sizeof(HW_STR) - strlen(HW_STR), 
				"\n%s",
				strAnalogModules.c_str());
		}

		//new line at the end of the line
		wp_snprintf(&HW_STR[strlen(HW_STR)],
			sizeof(HW_STR) - strlen(HW_STR),
			"\n");
		//move to next index in the vector
		hwIterator++;
	};

	strProbeList = HW_STR;

	wp_get_port_count_summary(strPortSummary);

	strProbeList += strPortSummary;

	return 0;
}

////////////////////////////////////////////////////////////////////////////////////////
// Public functions
////////////////////////////////////////////////////////////////////////////////////////
int wp_hwprobe::get_hwprobe_str(char *destination_buffer, int buffer_size, char verbose)
{
	std::string strProbeList;

	cVerbose = verbose;

	if(run(strProbeList)){
		return 1;
	}

	wp_snprintf(destination_buffer, buffer_size, "%s", strProbeList.c_str());	
	return 0;
}
